#version 430


//######################################################//
//### Standard inputs/uniforms for every pixelshader ###//
//######################################################//
layout(std140, binding = 0) uniform perFrame
{
	mat4x4	vp;
	mat4x4  invVP;

	mat4x4	view;
	mat4x4	invView;
	mat4x4	invTransView;

	mat4x4	proj;
	mat4x4	invProj;

	vec3	eyePos;
	float	gt;
	float	dt;
	uint	screenW;
	uint	screenH;

};

layout(location = 1) uniform mat4x4 modelToWorld;
//layout(location = 25) uniform mat4x4 modelInvTrans;

layout(location = 0) in vec4 iPosW;
layout(location = 1) in vec4 iPosH;
layout(location = 2) in vec3 iNormal;
layout(location = 3) in vec3 iTangent;
layout(location = 4) in vec3 iBitangent;
layout(location = 5) in vec2 iTex;

layout(location = 5) uniform vec3  iAlbedo;
layout(location = 6) uniform float iMetal;
layout(location = 7) uniform float iRoughness;
layout(location = 8) uniform vec3 iEmissive;

layout(location = 9) uniform bool hasAlbedoTexture;
layout(location = 10) uniform bool hasNormalTexture;
layout(location = 11) uniform bool hasMetalTexture;
layout(location = 12) uniform bool hasRoughnessTexture;
layout(location = 13) uniform bool hasEmissiveTexture;

layout(binding = 0) uniform sampler2D texAlbedo;
layout(binding = 1) uniform sampler2D texNormal;
layout(binding = 2) uniform sampler2D texMetal;
layout(binding = 3) uniform sampler2D texRoughness;
layout(binding = 4) uniform sampler2D texEmissive;

layout(location = 0) out vec4 oAlbedoMetal;
layout(location = 1) out vec4 oNormalDepth;
layout(location = 2) out vec4 oEmissiveRoughness;
//######################################################//
//### Standard inputs/uniforms for every pixelshader ###//
//######################################################//


void main()
{
	vec3 albedo;
	if( hasAlbedoTexture )
		albedo = texture(texAlbedo, iTex).zyx;
	else
		albedo = iAlbedo;

	vec3 normal = normalize(iNormal);
	if( hasNormalTexture )
	{
		vec3 N = (texture(texNormal, iTex).zyx  - .5) * 2;

		vec3 tangent = normalize(iTangent);
		vec3 bitangent = normalize(iBitangent);

		//Transform tangent-space vector N to world-space
		// https://gamedev.stackexchange.com/a/57477

		//tangent, bitangent, normal are in world-space
		// -> transforming tangent-space N with it gets you an world-space N
		mat3 toWorld = mat3(tangent, bitangent, normal);
		normal = normalize(toWorld * N);
	}

	float metal;
	if( hasMetalTexture )
		metal = texture(texMetal, iTex).x;
	else
		metal = iMetal;

	float roughness;
	if( hasRoughnessTexture )
		roughness = texture(texRoughness, iTex).x;
	else
		roughness = iRoughness;

	vec3 emissive;
	if( hasEmissiveTexture )
		emissive = texture(texEmissive, iTex).zyx;
	else
		emissive  = iEmissive;


	float depth = iPosH.z / iPosH.w;
//	float depth = iPosH.z;

	oAlbedoMetal = vec4(albedo, metal);
	oNormalDepth = vec4(normal, depth);
	oEmissiveRoughness = vec4(emissive, roughness);
}
