#version 430


//######################################################//
//### Standard inputs/uniforms for every pixelshader ###//
//######################################################//
layout(std140, binding = 0) uniform perFrame
{
	mat4x4	vp;
	mat4x4  invVP;

	mat4x4	view;
	mat4x4	invView;
	mat4x4	invTransView;

	mat4x4	proj;
	mat4x4	invProj;

	vec3	eyePos;
	float	gt;
	float	dt;
	uint	screenW;
	uint	screenH;

};

layout(location = 1) uniform mat4x4 modelToWorld;
//layout(location = 25) uniform mat4x4 modelInvTrans;

//INPUTS from Vertexshader
layout(location = 0) in vec4 iPosW;
layout(location = 1) in vec4 iPosH;
layout(location = 2) in vec2 iTex;

//INPUT uniforms used incase no textures are set
layout(location = 5) uniform vec3  iAlbedo;
layout(location = 6) uniform float iMetal;
layout(location = 7) uniform float iRoughness;
layout(location = 8) uniform vec3 iEmissive;
layout(location = 13) uniform float ALPHA = 1;

layout(location = 0) out vec4 oAlbedoMetal;
layout(location = 1) out vec4 oNormalDepth;
layout(location = 2) out vec4 oEmissiveRoughness;
//######################################################//
//### Standard inputs/uniforms for every pixelshader ###//
//######################################################//


//from: https://www.enkisoftware.com/devlogpost-20150131-1-Normal-generation-in-the-pixel-shader
vec3 calculateNormal(vec3 posW)
{
	vec3 dFdxPos = dFdx( posW );
    vec3 dFdyPos = dFdy( posW );
    return( normalize(cross(dFdxPos,dFdyPos)) );
}


void main()
{
	vec3 albedo = iAlbedo;

	vec3 posW = iPosW.xyz / iPosW.w;
	vec3 normal = -calculateNormal(posW);
	float metal = iMetal;
	float roughness = iRoughness;
	vec3 emissive  = iEmissive;


	float depth = iPosH.z / iPosH.w;

	oAlbedoMetal = vec4(albedo, metal);
	oNormalDepth = vec4(normal, depth);
	oEmissiveRoughness = vec4(emissive, roughness);
}
